@@ -599,8 +599,6 @@ static gpasmVal do_config(gpasmVal r,
 
   state.lst.line.linetype = config;
 
-  p = HEAD(parms);
-
   switch(arity) {
   case 1:
     if(_16bit_core) {
@@ -609,12 +607,12 @@ static gpasmVal do_config(gpasmVal r,
     } else {
       ca = state.device.config_address;
     }
+    p = HEAD(parms);
     break;
 
   case 2:
-    ca = maybe_evaluate(p);
+    ca = maybe_evaluate(HEAD(parms));
     p = HEAD(TAIL(parms));
-
     break;
 
   default:
@@ -2945,6 +2943,7 @@ struct insn op_0[] = {
   { "variable",   0, (long int)do_variable,  INSN_CLASS_FUNC,   0 },
   { "while",      0, (long int)do_while,     INSN_CLASS_FUNC,   0 },
   { ".set",       0, (long int)do_set, 	     INSN_CLASS_FUNC,   0 },
+  { "#if",        0, (long int)do_if,        INSN_CLASS_FUNC,   ATTRIB_COND },
   { "#else",      0, (long int)do_else,	     INSN_CLASS_FUNC,   ATTRIB_COND },
   { "#endif",     0, (long int)do_endif,     INSN_CLASS_FUNC,   ATTRIB_COND },
   { "#ifdef",     0, (long int)do_ifdef,     INSN_CLASS_FUNC,   ATTRIB_COND },
