@@ -49,7 +49,7 @@ select_processor(void)
   } else {
     printf("Didn't find any processor named: %s\nHere are the supported processors:\n",
             processor_name);
-    gp_dump_processor_list();
+    gp_dump_processor_list(true, 0);
     exit(1);
   }
 
@@ -208,7 +208,7 @@ int main(int argc, char *argv[])
       print_hex_info = 1;
       break;
     case 'l':
-      gp_dump_processor_list();
+      gp_dump_processor_list(true, 0);
       exit(0);
       break;
     case 'm':
