@@ -1,5 +1,6 @@
 /* Convert one word from memory into an equivalent assembly instruction
-   Copyright (C) 2001 Craig Franklin
+   Copyright (C) 2001, 2002, 2003, 2004
+   Craig Franklin
 
 This file is part of gputils.
 
@@ -21,7 +22,6 @@ Boston, MA 02111-1307, USA.  */
 #ifndef __GPDIS_H__
 #define __GPDIS_H__
 
-void mem2asm12(int insn, char buffer[80]);
-void mem2asm14(int insn, char buffer[80]);
+void gp_disassemble(MemBlock *m, int *org, enum proc_class class, char *buffer);
 
 #endif
